///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains the base class.
 *	\file		IceCell.h
 *	\author		Pierre Terdiman
 *	\date		February, 5, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICECELL_H__
#define __ICECELL_H__

	// Loop through all owners
	#define FOREACH_OWNER(cell, owner)										\
		{for(udword __idx=0;__idx<cell->GetNbOwners();__idx++){				\
			owner = cell->GetOwner(__idx);									\
			if(owner){														\

	// Loop through all references
	#define FOREACH_REF(cell, ref)											\
		{for(udword __idx=0;__idx<cell->GetNbRefs();__idx++){				\
			ref = cell->GetRef(__idx);										\
			if(ref){														\

	// Loop through all typed references
	#define FOREACH_TREF(cell, ref, type)									\
		{for(udword __idx=0;__idx<cell->GetNbRefs();__idx++){				\
			ref = cell->GetRef(__idx);										\
			if(ref && ref->GetType()==type){								\

	// Loop through all registered cells
	#define FOREACH_CELL(cell, type)										\
		{for(udword __idx=0;__idx<GetKernel()->GetNbEntries();__idx++){		\
			cell = GetKernel()->GetEntries()[__idx];						\
			if(cell && cell->GetType()==type){								\

	// End loop
	#define END_FOREACH	}}}

	//! Cell flags
	enum ICECORE_API CellFlag
	{
		CF_VALID			=	(1<<0),		//!< Cell has successfully been initialized
		CF_KERNEL_MODIF		=	(1<<1),		//!< Kernel control: enable direct cell members modifications by the kernel
		CF_KERNEL_DESTRUCT	=	(1<<2),		//!< Kernel control: enable direct cell destruction by the kernel
		CF_KERNEL_MSGS		=	(1<<3),		//!< Enable kernel messages
		CF_FORCE_DWORD		=	0x7fffffff
	};

	class Kernel;
	class DataBlock;

	class ICECORE_API Cell
	{
		protected:
		// Constructor/Destructor
		//! TO BE DOCUMENTED
										Cell();
		//! TO BE DOCUMENTED
		virtual							~Cell();

		public:
		//! TO BE DOCUMENTED
		virtual			udword			Import(ImportContext& ic);
		//! TO BE DOCUMENTED
		virtual			udword			Export(ExportContext& ec);
		//! TO BE DOCUMENTED
		virtual			bool			GetFields(Container& edit, udword flags);
		virtual			bool			GetFields(Container& edit, eFieldType type);
		virtual			bool			GetFields(Container& edit, eFieldType type, udword user);
		virtual			bool			GetFields(Container& edit);
		virtual			FieldDescriptor* GetFieldDescriptor(eFieldType type, udword user);
		//! TO BE DOCUMENTED
		virtual			bool			Message(IceMsg& msg)									{ return true;							}
						bool			SendMsgToOwners(IceMsg& msg);
						bool			SendMsgToRefs(IceMsg& msg);

		//! TO BE DOCUMENTED
						bool			CreateRef(Cell* ref);
						bool			CreateRef(KID kid);
						bool			CreateTypedRef(Cell* ref, RTYPE type);
		//! TO BE DOCUMENTED
						bool			DeleteRef(Cell* ref);
						bool			DeleteRef(KID kid);
		//! TO BE DOCUMENTED
						bool			ReplaceRef(Cell* oldref, Cell* newref);

		//! TO BE DOCUMENTED
						udword			GetNbRefs();
		//! TO BE DOCUMENTED
						udword			GetNbOwners();

		//! TO BE DOCUMENTED
						Cell*			GetOwner(udword i=0);
		//! TO BE DOCUMENTED
						Cell*			GetRef(udword i=0);

		//! TO BE DOCUMENTED
		__forceinline	uword			GetKernelID()											{ return m__KernelID;					}
		__forceinline	uword			GetType()												{ return m__Type;						}
		__forceinline	Cell&			SetType(udword type)									{ m__Type = type;		return *this;	}

		__forceinline	uword			GetFlags()												{ return m__Flags;						}
		__forceinline	bool			IsSet(CellFlag flag)									{ return (m__Flags&flag)!=0;			}
		__forceinline	Cell&			Enable(CellFlag flag)									{ m__Flags |= flag;		return *this;	}
		__forceinline	Cell&			Disable(CellFlag flag)									{ m__Flags &= ~flag;	return *this;	}

		//! Shared data blocks
						DataBlock*		CreateDataBlock(udword length);
						DataBlock*		ShareDataBlock(DataBlock* block);

		//! TO BE DOCUMENTED
						Cell&			SetName(String& name)									{ m__Name = name; return *this;			}
						Cell&			GetName(String& name)									{ name = m__Name; return *this;			}
		__forceinline	char*			GetName()												{ return m__Name.mText;					}

		//! TO BE DOCUMENTED
		virtual			bool			CanDestruct()											{ return GetNbOwners()==0;				}
		virtual			bool			SelfDestruct(udword context=0, udword userdata=0)		{ delete this;	return true;			}

		//! TO BE DOCUMENTED
						Cell*			Clone(bool copymembers=true);

		//! TO BE DOCUMENTED
						CLASS_DESCRIPTOR;
						DECLARE_CLASS(Cell);
		public:
		//! No commands defined for Cell
		virtual			bool			ExecCommand(const char* command, udword userdata=0)		{ return false;							}
		virtual			Cell&			EnumCommand(const char* command, udword userdata=0)		{ return *this;							}
						bool			DispatchCommand(const char* command, udword userdata=0, bool refs=true);
		private:
						KID				m__KernelID;	//!< Global unique kernel identifier
						uword			m__Flags;		//!< Cell flags
						udword			m__Type;		//!< Cell type
						String			m__Name;		//!< Cell name

		virtual			Cell*			CreateInstance()										{ return new Cell;						}

				friend	class			Kernel;
	};


	class ICECORE_API DataBlock : public Cell
	{
		public:
										// Constructor/Destructor
										DataBlock();
		virtual							~DataBlock();

						DECLARE_FIELDS(DataBlock, Cell);

			// Initialize
						bool			Init(udword length)										{ return mBlock.Init(length);			}

						DynamicBlock	mBlock;
	};

#endif // __ICECELL_H__
