///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Main file for IceCore.dll.
 *	\file		IceCore.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICECORE_H__
#define __ICECORE_H__

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Compilation messages

#ifndef __cplusplus
	#error ICE needs C++ compilation...
#endif

#if defined(ICECORE_EXPORTS)
	#pragma message("Compiling ICE Core")
#elif !defined(ICECORE_EXPORTS)
	#pragma message("Using ICE Core")
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes

// Windows includes
#ifdef WIN32
#include <windows.h>												// General purpose Win32 functions
#include <windowsx.h>
#endif // WIN32

// DOS includes
#ifdef MSDOS
#include <dos.h>
#define DIR_STRUCT struct find_t
#define FIND_FIRST( n, d, a ) _dos_findfirst( n, a, d )
#define FIND_NEXT _dos_findnext
#define DIR_FILE_NAME name
#endif // MSDOS

// VMS includes
#ifdef VMS
#include <ssdef.h>
#include <rmsdef.h>
#include <descrip.h>
#endif // VMS

// Standard includes
#ifdef _DEBUG
#include <crtdbg.h>													// C runtime debug functions
#endif
#include <stdio.h>													// Standard Input/Output functions
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>													// Assertion functions
#include <shlobj.h>
#include <mmsystem.h>
#include <stdarg.h>
#include <time.h>
#include <float.h>

// ICE includes
	#include "IcePreprocessor.h"
	#include "IceTypes.h"
	#include "IceDynID.h"
	#include "IceCPUID.h"
	#include "IceFPU.h"
	#include "IceMemoryManager.h"
	#include "IceDll.h"
	#include "IceQSort.h"
	#include "IceNLogLogN.h"

	namespace IceCore
	{
		#include "IceInstrumentTrace.h"
		#include "IceString.h"
		#include "IceMessage.h"
		#include "IceError.h"
		#include "IceKeyboard.h"
		#include "IceFile.h"
		#include "IceBuffer.h"

		#include "IceContainer.h"
		#include "IceCustomArray.h"
		#include "IceDynamicList.h"

//		#include "IceQSort.h"
		#include "IceRevisitedRadix.h"

		#include "IceProfiler.h"
		#include "IceImportExport.h"
		#include "IceRefTracker.h"
		#include "IceCell.h"
		#include "IceCustomList.h"
		#include "IceKernel.h"

		#include "IceRegistry.h"
	}

	using namespace IceCore;

	// Log message
	ICECORE_API void	Log(ubyte value);
	ICECORE_API void	Log(uword value);
	ICECORE_API void	Log(udword value);
	ICECORE_API void	Log(float value);
	ICECORE_API void	Log(sbyte value);
	ICECORE_API void	Log(sword value);
	ICECORE_API void	Log(sdword value);
	ICECORE_API void	Log(const char* msg);
	ICECORE_API	void	Log(LPSTR fmt, ...);

	ICECORE_API bool	CheckSingleRun(const char* appname);
	ICECORE_API void	Dump(const char* filename, const char* text);

	//! A function to test the DLL components.
	ICECORE_API void	TestIceCore();

#endif // __ICECORE_H__

