///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains a dynamic list class.
 *	\file		IceDynamicList.h
 *	\author		Pierre Terdiman
 *	\date		December, 12, 1999
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEDYNAMICLIST_H__
#define __ICEDYNAMICLIST_H__

	//! Initial list size
	#define DEFAULT_DYNAMICLIST_SIZE		2

	class ICECORE_API DynamicList
	{
		public:
		// Constructor / Destructor
										DynamicList();
										~DynamicList();

		// Basic usage
						uword			Add(udword value);
						void			Remove(uword virtualindex);

		// Physical data access
		__forceinline	udword			GetMaxNbEntries()	{ return mMaxNbEntries;		}	//!< Returns max number of entries		
		__forceinline	udword			GetNbEntries()		{ return mCurrentNbEntries;	}	//!< Returns current number of entries		
		__forceinline	udword*			GetList()			{ return mList;				}	//!< Gets the complete list
						udword			GetValue(uword virtualindex);						// Returns real value according to virtual index.

		//! High-speed access - same as GetValue without any checkings - handle with care.
		__forceinline	udword			PickValue(uword virtualindex)	{ return mList[mOutToIn[virtualindex]]; }

			// Processing
						bool			Foreach(ENUMERATION proc, udword param=0, udword context=0);
		protected:
		// Physical list
						udword*			mList;												//!< Physical list, with no holes but unsorted.
						uword			mCurrentNbEntries;									//!< Current number of values in the physical list.
						uword			mMaxNbEntries;										//!< Maximum possible number of values in the physical list.

		// Cross-references
						uword*			mOutToIn;											//!< Maps virtual indices to real ones.
						uword*			mInToOut;											//!< Maps real indices to virtual ones.

		// Recycled locations
						uword			mNbFreeIndices;										//!< Current number of free indices
	};

#endif //__ICEDYNAMICLIST_H__


