///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains error-related code.
 *	\file		IceError.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEERROR_H__
#define __ICEERROR_H__

	class ICECORE_API IceError
	{
	public:
				IceError();
				~IceError();

		String	mErrorText;				//!< A string describing the last error
		DynID	mErrorCode;				//!< A possible error code (-1 is undefined, 0 should be ok)
		udword	mErrorTimeStamp;		//!< A timestamp for the last error
	};

ICECORE_API void		SetIceErrorMsg(const char* error, DynID errorcode=-1, bool updatelog=true);
ICECORE_API IceError*	GetLastIceError();

#define	LASTERRORTEXT	GetLastIceError()->mErrorText							//!< Laziness...
#define	LASTERRORCODE	GetLastIceError()->mErrorCode							//!< Laziness...


// Following macros mimic the exception mechanism, without needing exception support (useful on Dreamcast).
// Standard usage:
//
//	ICE_TRY		Subroutine
//	ICE_CATCH	MessageBox(null, LASTERRORTEXT, "Oops", MB_OK);
//
// That's what I call "ICEptions" ...!
//
#define	ICE_TRY			udword __RDTSEC = GetLastIceError()->mErrorTimeStamp;	//!< Read time stamp error counter
#define ICE_CATCH		if(GetLastIceError()->mErrorTimeStamp!=__RDTSEC)		//!< Compare to previous counter

#endif // __ICEERROR_H__
