///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains the reference tracking system.
 *	\file		IceRefTracker.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICEREFTRACKER_H__
#define __ICEREFTRACKER_H__

	// Tracker flags
	#define	REFTRACK_INVALIDOFFSETS		(1<<0)		//!< Offsets must be recomputed (lazy evaluation)

	class ICECORE_API RefTracker
	{
		public:
		// Constructor/Destructor
										RefTracker();
										~RefTracker();

						bool			AddRef(udword owner, udword ref, RTYPE type=0);
						bool			RemoveRef(udword owner, udword ref);
						bool			RemoveIndex(udword id);

		__forceinline	udword*			GetOwnersList()						{	return mOwners;							}
		__forceinline	udword*			GetRefsList()						{	return mRefs;							}
		__forceinline	udword			GetNbPairs()						{	return mCurNbPairs;						}

		__forceinline	udword			GetNbMarkedIndices()				{	return mMarkedIndices.GetNbEntries();	}
		__forceinline	udword*			GetMarkedIndices()					{	return mMarkedIndices.GetEntries();		}

		__forceinline	udword			GetNbMarkedLinks()					{	return mMarkedLinks.GetNbEntries();		}
		__forceinline	udword*			GetMarkedLinks()					{	return mMarkedLinks.GetEntries();		}

						udword			GetNbRefs(udword owner);
						udword			GetNbOwners(udword ref);

						udword			GetOwner(udword ref, udword i);
						udword			GetRef(udword owner, udword i);

						bool			SetType(udword owner, udword ref, RTYPE type);

						udword			GetUsedRam();

		// Dump
						RefTracker&		Dump();

		private:
						udword			mFlags;

						udword			mMaxNbPairs;
						udword			mCurNbPairs;
						udword*			mOwners;
						udword*			mRefs;

						udword*			mSortedOwners;
						udword*			mSortedRefs;

						udword*			mOwnersOffsets;
						udword*			mRefsOffsets;

						udword			mMinOwner;
						udword			mMinRef;
						udword			mMaxOwner;
						udword			mMaxRef;

						Container		mMarkedLinks;
						Container		mMarkedIndices;

		// Methods
						bool			CreateOffsets();
						bool			PurgeList();
						udword			GetPairIndex(udword owner, udword ref);
						bool			MarkLink(udword owner, udword ref);
						bool			MarkIndex(udword id);
	};

#endif // __ICEREFTRACKER_H__
