///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains source code from the article "Radix Sort Revisited".
 *	\file		IceRevisitedRadix.h
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Include Guard
#ifndef __ICERADIXSORT_H__
#define __ICERADIXSORT_H__


	class ICECORE_API RadixSorter
	{
		public:
		// Constructor/Destructor
										RadixSorter();
										~RadixSorter();

		// Sorting methods
						RadixSorter&	Sort(udword* input, udword nb, bool signedvalues=true);
						RadixSorter&	Sort(float* input, udword nb);

		//! Access to results. mIndices is a list of indices in sorted order, i.e. in the order you may further process your data
		__forceinline	udword*			GetIndices()				{ return mIndices; }

		// Reset the inner indices
						RadixSorter&	ResetIndices();

		// Stats
						udword			GetUsedRam();
		//! Returns the total number of calls to the radix sorter.
		__forceinline	udword			GetNbTotalCalls()			{ return mTotalCalls;	}
		//! Returns the number of premature exits due to temporal coherence.
		__forceinline	udword			GetNbHits()					{ return mNbHits;		}

		private:
						udword*			mHistogram;					//!< Counters for each byte
						udword*			mOffset;					//!< Offsets (nearly a cumulative distribution function)

						udword			mCurrentSize;				//!< Current size of the indices list
						udword*			mIndices;					//!< Two lists, swapped each pass
						udword*			mIndices2;

						// Stats
						udword			mTotalCalls;
						udword			mNbHits;
	};

#endif // __ICERADIXSORT_H__
