///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 *	Contains misc buffer related code.
 *	\file		IceBuffer.cpp
 *	\author		Pierre Terdiman
 *	\date		April, 4, 2000
 */
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Precompiled Header
#include "Stdafx.h"

using namespace IceCore;


ICECORE_API bool IceCore::Delta(void* buffer, udword nbitems, udword itemsize)
{
	switch(itemsize)
	{
		case 1:
		{
			sbyte* Buf = (sbyte*)buffer;
			sbyte Previous = Buf[0];
			for(udword i=1;i<nbitems;i++)
			{
				sbyte Saved = Buf[i];
				Buf[i]-=Previous;
				Previous = Saved;
			}
			return true;
		}
		break;

		case 2:
		{
			sword* Buf = (sword*)buffer;
			sword Previous = Buf[0];
			for(udword i=1;i<nbitems;i++)
			{
				sword Saved = Buf[i];
				Buf[i]-=Previous;
				Previous = Saved;
			}
			return true;
		}
		break;

		case 4:
		{
			sdword* Buf = (sdword*)buffer;
			sdword Previous = Buf[0];
			for(udword i=1;i<nbitems;i++)
			{
				sdword Saved = Buf[i];
				Buf[i]-=Previous;
				Previous = Saved;
			}
			return true;
		}
		break;
	}
	return false;
}

ICECORE_API bool IceCore::UnDelta(void* buffer, udword nbitems, udword itemsize)
{
	switch(itemsize)
	{
		case 1:
		{
			sbyte* Buf = (sbyte*)buffer;
			for(udword i=1;i<nbitems;i++)	Buf[i]+=Buf[i-1];
			return true;
		}
		break;

		case 2:
		{
			sword* Buf = (sword*)buffer;
			for(udword i=1;i<nbitems;i++)	Buf[i]+=Buf[i-1];
			return true;
		}
		break;

		case 4:
		{
			sdword* Buf = (sdword*)buffer;
			for(udword i=1;i<nbitems;i++)	Buf[i]+=Buf[i-1];
			return true;
		}
		break;
	}
	return false;
}



/*

  Input buffer is made of "nbentries" whose size is "stride".

*/
ICECORE_API bool IceCore::DisruptBuffer(void* buffer, udword nbentries, udword itemsize, udword stride)
{
	// Create a local copy
	ubyte* Copy = new ubyte[nbentries*stride];
	CHECKALLOC(Copy);
	CopyMemory(Copy, buffer, nbentries*stride*sizeof(ubyte));

	ubyte* dest = (ubyte*)buffer;
	udword NbCol = stride / itemsize;
	for(udword j=0;j<NbCol;j++)
	{
		ubyte* src = Copy;
		src += j*itemsize;
		for(udword i=0;i<nbentries;i++)
		{
			CopyMemory(dest, src, itemsize*sizeof(ubyte));
			src += stride;
			dest += itemsize;
		}
	}

	return true;
}

ICECORE_API bool IceCore::SaveAsSource(String& filename, String& name, void* buffer, udword length)
{
	if(length&3)	return false;
	length>>=2;

	CustomArray Array;
	Array.StoreASCII("udword ").StoreASCII(name.mText).StoreASCII("[]={\n");

	ubyte ConvTable[]= { '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f' };

	ubyte* Buf = (ubyte*)buffer;
	for(udword i=0;i<length;i++)
	{
		char Text[32];
		ubyte c3 = Buf[i*4+0];
		ubyte c2 = Buf[i*4+1];
		ubyte c1 = Buf[i*4+2];
		ubyte c0 = Buf[i*4+3];

		ubyte o=0;
		Text[o++] = ConvTable[c0>>4];
		Text[o++] = ConvTable[c0&15];

		Text[o++] = ConvTable[c1>>4];
		Text[o++] = ConvTable[c1&15];

		Text[o++] = ConvTable[c2>>4];
		Text[o++] = ConvTable[c2&15];

		Text[o++] = ConvTable[c3>>4];
		Text[o++] = ConvTable[c3&15];

		Text[o++] = 0;

		if(c0 || c1 || c2 || c3)	Array.StoreASCII("0x").StoreASCII(Text);
		else						Array.StoreASCII("0");

		if(i!=length-1)				Array.StoreASCII(", ");
		if(!(i&15))					Array.StoreASCII("\n");
	}

	Array.StoreASCII("\n};");
	Array.ExportToDisk(filename.mText);
	return true;
}
